from musicscore.musictree.midi import *
from musicscore.musictree.treechordflags import *
from musicscore.musictree.treechordflags2 import *
from musicscore.musictree.treechordflags3 import *
from musicscore.musictree.treebeat import *
from musicscore.musictree.treechord import *
from musicscore.musictree.treeclef import *
from musicscore.musictree.treeinstruments import *
from musicscore.musictree.treemeasure import *
from musicscore.musictree.treenote import *
from musicscore.musictree.treepagestyle import *
from musicscore.musictree.treepart import *
from musicscore.musictree.treescorepart import *
from musicscore.musictree.treescoretimewise import *
from musicscore.musictree.treetime import *



from musicscore.musicxml.types.complextypes.attributes import Clef
from musicscore.musicxml.types.complextypes.clef import Sign, Line, ClefOctaveChange


class TreeClef(Clef):

    def __init__(self, sign='G', line=2, octave_change=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._sign = None
        self._line = None
        self._octave_change = None

        self.sign = sign
        self.line = line
        self.octave_change = octave_change

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.sign == other.sign and self.line == other.line and self.octave_change == other.octave_change
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    @property
    def sign(self):
        return self._sign

    @sign.setter
    def sign(self, value):
        if not self._sign:
            self._sign = self.add_child(Sign(value))
        else:
            self._sign.value = value

        self._sign = value

    @property
    def line(self):
        return self._line

    @line.setter
    def line(self, value):
        if not self._line:
            self._line = self.add_child(Line(value))
        else:
            self._line.value = value

        self._line = value

    @property
    def octave_change(self):
        return self._octave_change

    @octave_change.setter
    def octave_change(self, value):
        if value:
            if not self._octave_change:
                self._octave_change = self.add_child(ClefOctaveChange(value))
            else:
                self._octave_change.value = value

            self._octave_change = value
        else:
            if self._octave_change:
                self.remove_child(self._octave_change)
                self._octave_change = None


SUPER_HIGH_TREBLE_CLEF = TreeClef(sign='G', line=2, octave_change=2)
HIGH_TREBLE_CLEF = TreeClef(sign='G', line=2, octave_change=1)
TREBLE_CLEF = TreeClef(sign='G', line=2)
LOW_TREBLE_CLEF = TreeClef(sign='G', line=2, octave_change=-1)
BASS_CLEF = TreeClef(sign='F', line=4)
LOW_BASS_CLEF = TreeClef(sign='F', line=4, octave_change=-1)
SUPER_LOW_BASS_CLEF = TreeClef(sign='F', line=4, octave_change=-2)

ALTO_CLEF = TreeClef(sign='C', line=3)
TENOR_CLEF = TreeClef(sign='C', line=4)

PERCUSSION_CLEF = TreeClef('percussion')



from unittest import TestCase
import os

from musicscore.musicstream import SimpleFormat
from musicscore.musictree.treechordflags import BeatwiseFlag
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from tests.score_templates.xml_test_score import TestScore

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        xml_path = path + '_test_1.xml'
        durations = [5]
        self.score.set_time_signatures(times={1: (5, 4)})
        sf = SimpleFormat(durations=durations)

        sf.to_stream_voice().add_to_score(self.score, part_number=1)
        sf.chords[0].add_flag(BeatwiseFlag())
        sf.to_stream_voice().add_to_score(self.score, part_number=2)

        self.score.write(xml_path)
        TestScore().assert_template(xml_path)

    def test_2(self):
        # todo
        xml_path = path + '_test_2.xml'
        durations = [4, 5, 6, 7, 8, 9, 10]
        self.score.set_time_signatures(
            times={1: (4, 4), 2: (5, 4), 3: (6, 4), 4: (7, 4), 5: (8, 4), 6: (9, 4), 7: (10, 4)})
        sf = SimpleFormat(durations=durations)

        sf.to_stream_voice().add_to_score(self.score, part_number=1)
        for chord in sf.chords:
            chord.add_flag(BeatwiseFlag())
        sf.to_stream_voice().add_to_score(self.score, part_number=2)

        self.score.write(xml_path)
        # TestScore().assert_template(xml_path)



from musicscore.dtd.dtd import Sequence, Element, GroupReference
from musicscore.musicxml.groups.common import Editorial
from musicscore.musicxml.elements.xml_element import XMLElement
from musicscore.musicxml.types.complextypes.complextype import ComplexType


class GroupName(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-name', *args, **kwargs)
        raise NotImplementedError('GroupName')


class GroupNameDisplay(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-name-display', *args, **kwargs)
        raise NotImplementedError('GroupNameDisplay')


class GroupAbbreviation(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-abbreviation', *args, **kwargs)
        raise NotImplementedError('GroupAbbreviation')


class GroupAbbreviationDisplay(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-abbreviation-display', *args, **kwargs)
        raise NotImplementedError('GroupAbbreviationDisplay')


class GroupSymbol(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-symbol', *args, **kwargs)
        raise NotImplementedError('GroupSymbol')


class GroupTime(XMLElement):
    """"""

    def __init__(self, *args, **kwargs):
        super().__init__(tag='group-time', *args, **kwargs)
        raise NotImplementedError('GroupTime')


class ComplexTypePartGroup(ComplexType):
    """
    The part-group element indicates groupings of parts in the score, usually indicated by braces and brackets. Braces
    that are used for multi-staff parts should be defined in the attributes element for that part. The part-group start
    element appears before the first score-part in the group. The part-group stop element appears after the last
    score-part in the group.

    The number attribute is used to distinguish overlapping and nested part-groups, not the sequence of groups. As with
    parts, groups can have a name and abbreviation. Values for the child elements are ignored at the stop of a group.

    A part-group element is not needed for a single multi-staff part. By default, multi-staff parts include a brace
    symbol and (if appropriate given the bar-style) common barlines. The symbol formatting for a multi-staff part can
    be more fully specified using the part-symbol element.

		<xs:sequence>
			<xs:element name="group-name" type="group-name" minOccurs="0"/>
			<xs:element name="group-name-display" type="name-display" minOccurs="0">
				<xs:annotation>
					<xs:documentation>Formatting specified in the group-name-display element overrides formatting specified in the group-name element.</xs:documentation>
				</xs:annotation>
			</xs:element>
			<xs:element name="group-abbreviation" type="group-name" minOccurs="0"/>
			<xs:element name="group-abbreviation-display" type="name-display" minOccurs="0">
				<xs:annotation>
					<xs:documentation>Formatting specified in the group-abbreviation-display element overrides formatting specified in the group-abbreviation element.</xs:documentation>
				</xs:annotation>
			</xs:element>
			<xs:element name="group-symbol" type="group-symbol" minOccurs="0"/>
			<xs:element name="group-barline" type="group-barline" minOccurs="0"/>
			<xs:element name="group-time" type="empty" minOccurs="0">
				<xs:annotation>
					<xs:documentation>The group-time element indicates that the displayed time signatures should stretch across all parts and staves in the group.</xs:documentation>
				</xs:annotation>
			</xs:element>
			<xs:group ref="editorial"/>
		</xs:sequence>
		<xs:attribute name="type" type="start-stop" use="required"/>
		<xs:attribute name="number" type="xs:token" default="1"/>
	</xs:complexType>
    """
    _DTD = Sequence(
        Element(GroupName, min_occurrence=0),
        Element(GroupNameDisplay, min_occurrence=0),
        Element(GroupAbbreviation, min_occurrence=0),
        Element(GroupAbbreviationDisplay, min_occurrence=0),
        Element(GroupSymbol, min_occurrence=0),
        Element(GroupTime, min_occurrence=0),
        GroupReference(Editorial)

    )

    def __init__(self, type_, number=1, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._type = None
        self.type_ = type_
        self._number = None
        self.number = number
        raise NotImplementedError()


from unittest import TestCase
import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treechord import TreeChord
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicscore.musicxml.elements.note import Duration
from musicscore.musicxml.groups.musicdata import Direction
from musicscore.musicxml.types.complextypes.direction import DirectionType
from musicscore.musicxml.types.complextypes.directiontype import Words
from tests.score_templates.xml_test_score import TestScore

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(durations=[1, 1])

        sf.chords[0].add_words('first word')
        sf.chords[1].add_words('second word', font_family='DejaVu Sans', font_size=14, font_weight='bold',
                               font_style='italic')

        v = sf.to_stream_voice(1)
        v.add_to_score(self.score, 1, 1)
        result_path = path + '_test_1'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)

    def test_2(self):
        sf = SimpleFormat(durations=[1, 1])

        sf.chords[0].add_words('a')
        sf.chords[1].add_words('b')
        sf.chords[0].add_words('aa', relative_y=-15)

        v = sf.to_stream_voice(1)
        v.add_to_score(self.score, 1, 1)

        result_path = path + '_test_2'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)

    def test_3(self):
        sf = SimpleFormat(durations=[1.95, 2.05])

        sf.chords[0].add_words('a')
        sf.chords[1].add_words('b')
        sf.chords[0].add_words('aa', relative_y=-15)

        v = sf.to_stream_voice(1)
        v.add_to_score(self.score, 1, 1)

        result_path = path + '_test_3'
        self.score.write(path=result_path)
        # TestScore().assert_template(result_path=result_path)

    def test_4(self):
        sf = SimpleFormat(durations=[2])

        chord = sf.chords[0]
        d = chord.add_child(Direction(placement='above'))
        dt = d.add_child(DirectionType())
        dt.add_child(Words('A', font_size=10))
        dt.add_child(Words('B', font_size=20))

        v = sf.to_stream_voice().add_to_score(self.score)

        result_path = path + '_test_4'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)



from musicscore.musictree.midi import Midi
from unittest import TestCase


class TestMidi(TestCase):
    def test_midi(self):
        midi = Midi(60)
        result = '''<pitch>
  <step>C</step>
  <octave>4</octave>
</pitch>
'''
        self.assertEqual(midi.get_pitch_rest().to_string(), result)
        midi = Midi(0)
        result = '''<rest/>
'''
        self.assertEqual(midi.get_pitch_rest().to_string(), result)
        midi = Midi(59, accidental_mode='enharmonic_1')
        result = '''<pitch>
  <step>C</step>
  <alter>-1</alter>
  <octave>4</octave>
</pitch>
'''
        self.assertEqual(midi.get_pitch_rest().to_string(), result)
        midi = Midi(59, accidental_mode='enharmonic_2')
        result = '''<pitch>
  <step>A</step>
  <alter>2</alter>
  <octave>3</octave>
</pitch>
'''
        self.assertEqual(midi.get_pitch_rest().to_string(), result)
        midi = Midi(58, accidental_mode='sharp')
        result = '''<pitch>
  <step>A</step>
  <alter>1</alter>
  <octave>3</octave>
</pitch>
'''
        self.assertEqual(midi.get_pitch_rest().to_string(), result)



import os
from unittest import TestCase

from musicscore.musicstream import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(durations=[3, 1])
        sf.chords[1].to_rest()
        sf.chords[0].add_tie('start')
        sf.chords[1].add_tie('stop')
        sf.chords[0].is_adjoinable = False
        xml_path = path + '_test_1.xml'
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)



from unittest import TestCase
import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from tests.score_templates.xml_test_score import TestScore

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(midis=[70, 72, 73], durations=[2, 0.5, 0.5])
        voice1 = sf.to_stream_voice(1)
        voice1.add_to_score(self.score, 1, 1)

        sf = SimpleFormat(midis=[50, 52, 53], durations=[0.5, 1, 2])
        voice2 = sf.to_stream_voice(2)
        voice2.add_to_score(self.score, 1, 1)

        result_path = path + '_test_1'
        self.score.write(result_path)
        TestScore().assert_template(result_path=result_path)



from unittest import TestCase

from musicscore.musicstream import SimpleFormat
from musicscore.musictree.treeinstruments import TreeInstrument, Violin, ViolaDamore
from musicscore.musictree.treescorepart import TreeScorePart
from musicscore.musictree.treescoretimewise import TreeScoreTimewise

import os

from tests.score_templates.xml_test_score import TestScore

path = os.path.abspath(__file__)


class Test(TestCase):
    def test_1(self):
        instrument = TreeInstrument(name='banjo', abbreviation='bjo', number=2)
        # score = TreeScoreTimewise()
        score_part = TreeScorePart(id='p1', instrument=instrument)
        # score.add_score_part()
        # violin = Violin(1)
        # score.add_score_part(TreeScorePart('p2', violin))
        # sf = SimpleFormat(durations=5 * [1, 2, 3, 4])
        # sf.to_stream_voice().add_to_score(score, 1, 1)
        # sf = SimpleFormat(durations=4 * [4, 3, 2])
        # sf.to_stream_voice().add_to_score(score, 1, 2)
        # xml_path = path.split('.')[0] + '.xml'
        # score.write(xml_path)
        # TestScore().assert_template(result_path=xml_path)

    def test_2(self):
        instrument = ViolaDamore()
        string = instrument.strings[3]
        # print(string.tuning)